@@ -52,10 +52,9 @@
 #include "animation.h"
 #endif
 
-#define STYLE_FUNCTION(function) (clearlooks_style_class->style_functions[CLEARLOOKS_STYLE (style)->style].function)
+#define STYLE_FUNCTION(function) (CLEARLOOKS_STYLE_GET_CLASS (style)->style_functions[CLEARLOOKS_STYLE (style)->style].function)
 
-static ClearlooksStyleClass *clearlooks_style_class;
-static GtkStyleClass *clearlooks_parent_class;
+G_DEFINE_DYNAMIC_TYPE (ClearlooksStyle, clearlooks_style, GTK_TYPE_STYLE)
 
 static void
 clearlooks_set_widget_parameters (const GtkWidget      *widget,
@@ -63,8 +62,8 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
                                   GtkStateType          state_type,
                                   WidgetParameters     *params)
 {
-	params->style_functions = &(clearlooks_style_class->style_functions[CLEARLOOKS_STYLE (style)->style]);
-	params->style_constants = &(clearlooks_style_class->style_constants[CLEARLOOKS_STYLE (style)->style]);
+	params->style_functions = &(CLEARLOOKS_STYLE_GET_CLASS (style)->style_functions[CLEARLOOKS_STYLE (style)->style]);
+	params->style_constants = &(CLEARLOOKS_STYLE_GET_CLASS (style)->style_constants[CLEARLOOKS_STYLE (style)->style]);
 
 	params->active        = (state_type == GTK_STATE_ACTIVE);
 	params->prelight      = (state_type == GTK_STATE_PRELIGHT);
@@ -72,7 +71,7 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
 	params->state_type    = (ClearlooksStateType)state_type;
 	params->corners       = CR_CORNER_ALL;
 	params->ltr           = ge_widget_is_ltr ((GtkWidget*)widget);
-	params->focus         = widget && GTK_WIDGET_HAS_FOCUS (widget);
+	params->focus         = !CLEARLOOKS_STYLE (style)->disable_focus && widget && GTK_WIDGET_HAS_FOCUS (widget);
 	params->is_default    = widget && GE_WIDGET_HAS_DEFAULT (widget);
 	params->enable_shadow = FALSE;
 	params->radius        = CLEARLOOKS_STYLE (style)->radius;
@@ -134,6 +133,25 @@ clearlooks_style_draw_flat_box (DRAW_ARGS)
 
 		cairo_destroy (cr);
 	}
+	else if (DETAIL ("icon_view_item"))
+	{
+		WidgetParameters params;
+		ClearlooksStyle  *clearlooks_style;
+		ClearlooksColors *colors;
+		cairo_t          *cr;
+
+		CHECK_ARGS
+		SANITIZE_SIZE
+
+		clearlooks_style = CLEARLOOKS_STYLE (style);
+		clearlooks_set_widget_parameters (widget, style, state_type, &params);
+		colors = &clearlooks_style->colors;
+		cr = ge_gdk_drawable_to_cairo (window, area);
+
+		STYLE_FUNCTION (draw_icon_view_item) (cr, colors, &params, x, y, width, height);
+
+		cairo_destroy (cr);
+	}
 	else if ((CLEARLOOKS_STYLE (style)->style == CL_STYLE_GLOSSY || CLEARLOOKS_STYLE (style)->style == CL_STYLE_GUMMY) &&
 	         ((DETAIL("checkbutton") || DETAIL("radiobutton")) && state_type == GTK_STATE_PRELIGHT))
 	{
@@ -141,7 +159,7 @@ clearlooks_style_draw_flat_box (DRAW_ARGS)
 	}
 	else
 	{
-		clearlooks_parent_class->draw_flat_box (style, window, state_type,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_flat_box (style, window, state_type,
 		                                        shadow_type,
 		                                        area, widget, detail,
 		                                        x, y, width, height);
@@ -182,6 +200,17 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 			else
 				params.corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
 		}
+		
+		/* Fill the background as it is initilized to base[NORMAL].
+		 * Relevant GTK+ bug: http://bugzilla.gnome.org/show_bug.cgi?id=513471
+		 * The fill only happens if no hint has been added by some application
+		 * that is faking GTK+ widgets. */
+		if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+		{
+			cairo_rectangle (cr, 0, 0, width, height);
+			ge_cairo_set_color (cr, &params.parentbg);
+			cairo_fill (cr);
+		}
 
 		STYLE_FUNCTION (draw_entry) (cr, &clearlooks_style->colors, &params,
 		                             x, y, width, height);
@@ -198,7 +227,7 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 			STYLE_FUNCTION (draw_statusbar) (cr, colors, &params,
 			                                 x, y, width, height);
 	}
-	else if (DETAIL ("frame"))
+	else if (DETAIL ("frame") || DETAIL ("calendar"))
 	{
 		WidgetParameters params;
 		FrameParameters  frame;
@@ -220,7 +249,7 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 		if (CLEARLOOKS_STYLE (style)->style == CL_STYLE_CLASSIC)
 			ge_shade_color ((CairoColor*)&colors->bg[0], 0.78, &border);
 		else
-			border = (CairoColor)colors->shade[5];
+			border = colors->shade[5];
 
 		cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);
 		ge_cairo_set_color (cr, &border);
@@ -334,7 +363,7 @@ clearlooks_style_draw_box_gap (DRAW_ARGS,
 	}
 	else
 	{
-		clearlooks_parent_class->draw_box_gap (style, window, state_type, shadow_type,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_box_gap (style, window, state_type, shadow_type,
 		                                       area, widget, detail,
 		                                       x, y, width, height,
 		                                       gap_side, gap_x, gap_width);
@@ -397,7 +426,7 @@ clearlooks_style_draw_extension (DRAW_ARGS, GtkPositionType gap_side)
 	}
 	else
 	{
-		clearlooks_parent_class->draw_extension (style, window, state_type, shadow_type, area,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_extension (style, window, state_type, shadow_type, area,
 		                                         widget, detail, x, y, width, height,
 		                                         gap_side);
 	}
@@ -494,7 +523,7 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		gint columns, column_index;
 		gboolean resizable = TRUE;
 
-		/* XXX: This makes unknown treeview header CL_ORDER_MIDDLE, in need for something nicer */
+		/* XXX: This makes unknown treeview header "middle", in need for something nicer */
 		columns = 3;
 		column_index = 1;
 
@@ -516,12 +545,11 @@ clearlooks_style_draw_box (DRAW_ARGS)
 
 		header.resizable = resizable;
 
+		header.order = 0;
 		if (column_index == 0)
-			header.order = params.ltr ? CL_ORDER_FIRST : CL_ORDER_LAST;
-		else if (column_index == columns-1)
-			header.order = params.ltr ? CL_ORDER_LAST : CL_ORDER_FIRST;
-		else
-			header.order = CL_ORDER_MIDDLE;
+			header.order |= params.ltr ? CL_ORDER_FIRST : CL_ORDER_LAST;
+		if (column_index == columns-1)
+			header.order |= params.ltr ? CL_ORDER_LAST : CL_ORDER_FIRST;
 
 		gtk_style_apply_default_background (style, window, FALSE, state_type, area, x, y, width, height);
 
@@ -530,6 +558,13 @@ clearlooks_style_draw_box (DRAW_ARGS)
 	}
 	else if (DETAIL ("button") || DETAIL ("buttondefault"))
 	{
+		/* We are already checking the default button with the
+		 * "clearlooks_set_widget_parameters" function, so we may occur
+		 * in drawing the button two times. Let's return then.
+		 */
+		if (DETAIL ("buttondefault"))
+			return;
+	
 		WidgetParameters params;
 		ShadowParameters shadow = { CR_CORNER_ALL, CL_SHADOW_NONE } ;
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
@@ -648,6 +683,16 @@ clearlooks_style_draw_box (DRAW_ARGS)
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
+		/* Fill the background as it is initilized to base[NORMAL].
+		 * Relevant GTK+ bug: http://bugzilla.gnome.org/show_bug.cgi?id=513476
+		 * The fill only happens if no hint has been added by some application
+		 * that is faking GTK+ widgets. */
+		if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+		{
+			cairo_rectangle (cr, 0, 0, width, height);
+			ge_cairo_set_color (cr, &params.parentbg);
+			cairo_fill (cr);
+		}
 		STYLE_FUNCTION(draw_progressbar_trough) (cr, colors, &params,
 		                                         x, y, width, height);
 	}
@@ -929,7 +974,7 @@ clearlooks_style_draw_box (DRAW_ARGS)
 	}
 	else
 	{
-		clearlooks_parent_class->draw_box (style, window, state_type, shadow_type, area,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_box (style, window, state_type, shadow_type, area,
 		                                   widget, detail, x, y, width, height);
 	}
 
@@ -1001,7 +1046,7 @@ clearlooks_style_draw_slider (DRAW_ARGS, GtkOrientation orientation)
 	}
 	else
 	{
-		clearlooks_parent_class->draw_slider (style, window, state_type, shadow_type, area,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_slider (style, window, state_type, shadow_type, area,
 		                                      widget, detail, x, y, width, height, orientation);
 	}
 
@@ -1161,7 +1206,7 @@ clearlooks_style_draw_shadow_gap (DRAW_ARGS,
 	}
 	else
 	{
-		clearlooks_parent_class->draw_shadow_gap (style, window, state_type, shadow_type, area,
+		GTK_STYLE_CLASS (clearlooks_style_parent_class)->draw_shadow_gap (style, window, state_type, shadow_type, area,
 		                                          widget, detail, x, y, width, height,
 		                                          gap_side, gap_x, gap_width);
 	}
@@ -1289,7 +1334,7 @@ clearlooks_style_init_from_rc (GtkStyle * style,
 {
 	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
 
-	clearlooks_parent_class->init_from_rc (style, rc_style);
+	GTK_STYLE_CLASS (clearlooks_style_parent_class)->init_from_rc (style, rc_style);
 
 	g_assert ((CLEARLOOKS_RC_STYLE (rc_style)->style >= 0) && (CLEARLOOKS_RC_STYLE (rc_style)->style < CL_NUM_STYLES));
 	clearlooks_style->style               = CLEARLOOKS_RC_STYLE (rc_style)->style;
@@ -1302,6 +1347,7 @@ clearlooks_style_init_from_rc (GtkStyle * style,
 	clearlooks_style->colorize_scrollbar  = CLEARLOOKS_RC_STYLE (rc_style)->colorize_scrollbar;
 	clearlooks_style->animation           = CLEARLOOKS_RC_STYLE (rc_style)->animation;
 	clearlooks_style->radius              = CLAMP (CLEARLOOKS_RC_STYLE (rc_style)->radius, 0.0, 10.0);
+	clearlooks_style->disable_focus       = CLEARLOOKS_RC_STYLE (rc_style)->disable_focus;
 
 	if (clearlooks_style->has_focus_color)
 		clearlooks_style->focus_color     = CLEARLOOKS_RC_STYLE (rc_style)->focus_color;
@@ -1319,7 +1365,7 @@ clearlooks_style_realize (GtkStyle * style)
 	double contrast;
 	int i;
 
-	clearlooks_parent_class->realize (style);
+	GTK_STYLE_CLASS (clearlooks_style_parent_class)->realize (style);
 
 	contrast = CLEARLOOKS_RC_STYLE (style->rc_style)->contrast;
 
@@ -1374,6 +1420,10 @@ clearlooks_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType st
 	CHECK_ARGS
 	SANITIZE_SIZE
 
+	/* Just return if focus drawing is disabled. */
+	if (clearlooks_style->disable_focus)
+		return;
+
 	cr = gdk_cairo_create (window);
 
 	clearlooks_set_widget_parameters (widget, style, state_type, &params);
@@ -1505,6 +1555,10 @@ clearlooks_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType st
 	{
 		focus.type = CL_FOCUS_TREEVIEW; /* Treeview without content is focused. */
 	}
+	else if (DETAIL("icon_view"))
+	{
+		focus.type = CL_FOCUS_ICONVIEW;
+	}
 	else
 	{
 		focus.type = CL_FOCUS_UNKNOWN; /* Custom widgets (Beagle) and something unknown */
@@ -1544,14 +1598,15 @@ clearlooks_style_copy (GtkStyle * style, GtkStyle * src)
 	cl_style->animation           = cl_src->animation;
 	cl_style->radius              = cl_src->radius;
 	cl_style->style               = cl_src->style;
+	cl_style->disable_focus       = cl_src->disable_focus;
 
-	clearlooks_parent_class->copy (style, src);
+	GTK_STYLE_CLASS (clearlooks_style_parent_class)->copy (style, src);
 }
 
 static void
 clearlooks_style_unrealize (GtkStyle * style)
 {
-	clearlooks_parent_class->unrealize (style);
+	GTK_STYLE_CLASS (clearlooks_style_parent_class)->unrealize (style);
 }
 
 static GdkPixbuf *
@@ -1743,6 +1798,12 @@ clearlooks_style_draw_render_icon (GtkStyle            *style,
 	return stated;
 }
 
+void
+clearlooks_style_register_types (GTypeModule *module)
+{
+  clearlooks_style_register_type (module);
+}
+
 static void
 clearlooks_style_init (ClearlooksStyle * style)
 {
@@ -1753,9 +1814,6 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 {
 	GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);
 
-	clearlooks_style_class = CLEARLOOKS_STYLE_CLASS (klass);
-	clearlooks_parent_class = g_type_class_peek_parent (klass);
-
 	style_class->copy             = clearlooks_style_copy;
 	style_class->realize          = clearlooks_style_realize;
 	style_class->unrealize        = clearlooks_style_unrealize;
@@ -1779,46 +1837,26 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 	style_class->draw_layout      = clearlooks_style_draw_layout;
 	style_class->render_icon      = clearlooks_style_draw_render_icon;
 
-	clearlooks_register_style_classic (&clearlooks_style_class->style_functions[CL_STYLE_CLASSIC],
-	                                   &clearlooks_style_class->style_constants[CL_STYLE_CLASSIC]);
+	clearlooks_register_style_classic (&klass->style_functions[CL_STYLE_CLASSIC],
+	                                   &klass->style_constants[CL_STYLE_CLASSIC]);
 
-	clearlooks_style_class->style_functions[CL_STYLE_GLOSSY] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_style_class->style_constants[CL_STYLE_GLOSSY] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
-	clearlooks_register_style_glossy (&clearlooks_style_class->style_functions[CL_STYLE_GLOSSY],
-	                                  &clearlooks_style_class->style_constants[CL_STYLE_GLOSSY]);
+	klass->style_functions[CL_STYLE_GLOSSY] = klass->style_functions[CL_STYLE_CLASSIC];
+	klass->style_constants[CL_STYLE_GLOSSY] = klass->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_glossy (&klass->style_functions[CL_STYLE_GLOSSY],
+	                                  &klass->style_constants[CL_STYLE_GLOSSY]);
 
-	clearlooks_style_class->style_functions[CL_STYLE_INVERTED] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_style_class->style_constants[CL_STYLE_INVERTED] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
-	clearlooks_register_style_inverted (&clearlooks_style_class->style_functions[CL_STYLE_INVERTED],
-	                                    &clearlooks_style_class->style_constants[CL_STYLE_INVERTED]);
+	klass->style_functions[CL_STYLE_INVERTED] = klass->style_functions[CL_STYLE_CLASSIC];
+	klass->style_constants[CL_STYLE_INVERTED] = klass->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_inverted (&klass->style_functions[CL_STYLE_INVERTED],
+	                                    &klass->style_constants[CL_STYLE_INVERTED]);
 
-	clearlooks_style_class->style_functions[CL_STYLE_GUMMY] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_style_class->style_constants[CL_STYLE_GUMMY] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
-	clearlooks_register_style_gummy (&clearlooks_style_class->style_functions[CL_STYLE_GUMMY],
-	                                 &clearlooks_style_class->style_constants[CL_STYLE_GUMMY]);
+	klass->style_functions[CL_STYLE_GUMMY] = klass->style_functions[CL_STYLE_CLASSIC];
+	klass->style_constants[CL_STYLE_GUMMY] = klass->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_gummy (&klass->style_functions[CL_STYLE_GUMMY],
+	                                 &klass->style_constants[CL_STYLE_GUMMY]);
 }
 
-GType clearlooks_type_style = 0;
-
-void
-clearlooks_style_register_type (GTypeModule * module)
+static void
+clearlooks_style_class_finalize (ClearlooksStyleClass *klass)
 {
-	static const GTypeInfo object_info =
-	{
-		sizeof (ClearlooksStyleClass),
-		(GBaseInitFunc) NULL,
-		(GBaseFinalizeFunc) NULL,
-		(GClassInitFunc) clearlooks_style_class_init,
-		NULL,         /* class_finalize */
-		NULL,         /* class_data */
-		sizeof (ClearlooksStyle),
-		0,            /* n_preallocs */
-		(GInstanceInitFunc) clearlooks_style_init,
-		NULL
-	};
-
-	clearlooks_type_style = g_type_module_register_type (module,
-	                                                     GTK_TYPE_STYLE,
-	                                                     "ClearlooksStyle",
-	                                                     &object_info, 0);
 }
